/*
  Persistence of Vision Ray Tracer Scene Description File

  Tree placement include for the hf2iso demo

  Uses the slope map to avoid the rocky parts of the landscape,
  combined with trace to allow placement only under the desired 
  slope and always above the water level.
  
  --   
  Jaime Vives Piqueres, 2007-2008.

*/

// fake isosurface tree
#declare tree=
union{
 isosurface{
  function{f_sphere(x,y,z,1) - f_granite(x*.25,y*.25,z*.25)*.9}
  max_gradient 3
  contained_by{sphere{0,2}}
  double_illuminate
 }
 isosurface{
  function{f_sphere(x,y,z,1) - f_granite(x*.125+2,y*.125,z*.125)*.9}
  max_gradient 3
  contained_by{sphere{0,2}}
  scale .9
 }
 scale .25
}
// or use POVTrees like this:
//#include "eucaliptus1.inc"
//#declare tree=object{eucaliptus_TREE}

// slope map function  
#declare f_hf_slope2=
function{
 pigment{
  image_map{tga slopemap interpolate 2}
  rotate 90*x
  #if (terrain_turbulence)
   translate rand_disp
   scale turb_scale warp{turbulence turb_amount octaves turb_octaves lambda turb_lambda} scale 1/turb_scale
   translate -rand_disp
  #end
  translate <-.5,0,-.5> // this
  scale terrain_size    // is
  rotate terrain_rot*y  // to match the actual terrain placement
 }
}

// start placing trees
#declare i=0;
#while (i<(nt*1))
 // find a proper placement
 #declare pos_found=false;
 #while (!pos_found) 
  #declare Norm=<0,0,0>;
  #declare Start=<-forest_width*.5+forest_width*rand(r_t),terrain_size.y*2,cam_pos.z+forest_depth*rand(r_t)>;
   #declare Inter=trace(terrain,Start,-1*y,Norm);
   // check conditions for placement
   #if (
    // above the water?
    (Inter.y>(Water_Height+terrain_size.y*.01)) 
    // allowed trace() normal?
    & (Norm.y>norm_allow) 
    )
    // restricted by low slope map values?
    #if (use_slopemap)
     #if ((f_hf_slope2(Start.x,0,Start.z).gray)<.5)
      #declare pos_found=true;
     #end
    #else
     #declare pos_found=true;
    #end
   #end
 #end
 // place the tree randomly rotated on the y axis
 object{
  tree
  rotate 360*rand(r_t)*y
  scale (.4+.6*rand(r_t))*trees_scale
  translate Inter
  texture{t_trees}
 }
 #declare i=i+1;
#end
