/*
  Persistence of Vision Ray Tracer Scene Description File

  hf2iso terrain include

  Creates an isosurface version of a heightfield, using a slope map 
  to place extra deformations on the high slope parts (rocks). It also
  uses general turbulence to avoid pixelization.
  
  --   
  Jaime Vives Piqueres, 2007-2008.

*/

// random displacement for turbulence
#declare rand_disp=10000*rand(r_trr);  
// turbulence params
#declare turb_amount=.5;
#declare turb_octaves=9;
#declare turb_lambda=2;
#declare turb_scale=100*<1,4,1>;

// function for the heightfield
#declare f_hf=
function{
 pigment{
  image_map{tga heightmap interpolate 2}
  rotate 90*x
  #if (terrain_turbulence)
   translate rand_disp
   scale turb_scale warp{turbulence turb_amount octaves turb_octaves lambda turb_lambda} scale 1/turb_scale
   translate -rand_disp
  #end
  scale terrain_size
 }
}

// function for the slope map
#declare f_hf_slope=
function{
 pigment{
  image_map{tga slopemap interpolate 2}
  rotate 90*x
  #if (terrain_turbulence)
   translate rand_disp
   scale turb_scale warp{turbulence turb_amount octaves turb_octaves lambda turb_lambda} scale 1/turb_scale
   translate -rand_disp
  #end
  scale terrain_size
 }
}

// pigment function for displacement on the high slope parts
#declare f_rocks=
function{
 pigment{
  p_rocks
  color_map{
   [0 rgb 0]
   [1 rgb 1]
  }
 }
}

// include terrain textures
#include "hf2iso_textures.inc"

// extract terrain height for use on the isosurface function
#declare tsy=terrain_size.y; 

// random rotation for the heightfield
// this was originally a camera rotation, but that way some things were more
// difficult, so I opted for using a terrain rotation based on the camera seed
#declare terrain_rot=360*rand(r_cam); 

// create isosurface terrain 
#declare terrain=
isosurface{ 
  #if (terrain_detail)
  function { 
   y
   // main heighfield for the terrain shape
   -f_hf(x,0,z).hf*tsy   
   // add detail dependant on slope
   -f_rocks(x,y,z).gray*f_hf_slope(x,0,z).gray*tsy*rocks_depth
  }
  #else
  function { 
   y
   -f_hf(x,0,z).hf*tsy
  }
  #end
  max_gradient terrain_maxg 
  accuracy .0001
  contained_by{
   box{
    <0,0,0>,
    <terrain_size.x,terrain_size.y+tsy*rocks_depth,terrain_size.z>
   }
  }
  texture{t_terrain}
  translate <-terrain_size.x*.5,0,-terrain_size.z*.5>
  rotate terrain_rot*y 
}

// terrain placement
object{terrain
}

// filling ground plane
plane{y,-.001 texture{t_sand}}
