/*
  Persistence of Vision Ray Tracer Scene Description File

  hf2iso terrain texture include

  Creates a vetical gradient texture composed of the sand/underwater
  texture and the terrain texture itself. The terrain texture is based
  on "slope y" to make the transition between the rocks texture and the 
  grass/soil texture. The rocks texture is also based on "slope y" to
  adapt the image map to the surface slope.
  
  Image maps not provided: go to www.mayang.com/textures and grab some
  of the stones, marbles or soils textures (better if they have no shadows)
  
  --   
  Jaime Vives Piqueres, 2007-2008.

*/

// rocks texture for high slope parts
#if (use_imagemaps)  // use your own!
#declare rocks_img="/home/jaime/raytracing/povray-3.6/scenes/include/mayang-textures/Nature/images/Soil and Earth/stony_earth_011713.JPG";
#declare t_rocks_base=
texture{
 pigment{image_map{jpeg rocks_img}}
 normal{bump_map{jpeg rocks_img bump_size 2}}
 translate -.5 scale terrain_size.y
 rotate 360*rand(r_ttx)*z
}
#declare t_rocks=
texture{
 slope x
 texture_map{
  [0.00 t_rocks_base rotate 0*y]
  [0.125 t_rocks_base rotate 22.5*y]
  [0.25 t_rocks_base rotate 45*y]
  [0.375 t_rocks_base rotate 67.5*y]
  [0.50 t_rocks_base rotate 90*y]
  [0.625 t_rocks_base rotate 112.5*y]
  [0.75 t_rocks_base rotate 135*y]
  [0.875 t_rocks_base rotate 157.5*y]
  [1.00 t_rocks_base rotate 180*y]
 }
}
#else // use POV-Ray pigments
#declare p_rocks=
pigment{
 gradient y turbulence 1 lambda 3 scale (terrain_size.y/2)
 color_map{
   [0.0 DarkTan]
   [0.2 Flesh]
   [0.4 Gray20+Orange*.5]
   [0.6 DarkBrown]
   [0.8 Gray60]
   [1.0 DarkTan]
 }
}
#declare t_rocks=
texture{
 pigment{
  cells
  pigment_map{
   [0 p_rocks]
   [1 p_rocks translate terrain_size.y*y]
  }
  turbulence .5 lambda 3
 }
}
#end

// sand texture for the terrain near the water level
#declare t_sand_base=
texture{
 pigment{
  granite turbulence 1 omega 1
  scale .1
  color_map{
   [0 White]
   [0.25 DarkTan]
   [.5 Flesh]
   [0.75 DarkBrown]
   [1 Orange]
  }
 }
}
#declare t_sand=
texture{
 slope y
 texture_map{
  [0.00 t_sand_base]
  [0.50 t_rocks]
//  [0.75 t_rocks]
  [1.00 t_sand_base]
 }
}

// grass/soil texture for the low slope parts and trees
#if (use_imagemaps)
#declare grass_img2="/home/jaime/raytracing/povray-3.6/scenes/include/mayang-textures/Plants/images/Repetitive Plants Textures/summer_grass_daisies_5140178.JPG";
#declare grass_img1="/home/jaime/raytracing/povray-3.6/scenes/include/mayang-textures/Plants/images/Repetitive Plants Textures/uneven_grass_030199.JPG";
#declare t_grass1=
texture{
 pigment{image_map{jpeg grass_img1} turbulence .5 lambda 3 omega 1}
 normal{bump_map{jpeg grass_img1 bump_size 1} turbulence .5 lambda 3 omega 1}
 translate -.5 rotate 90*x scale 10
}
#declare t_grass2=
texture{
 pigment{image_map{jpeg grass_img2} turbulence .5 lambda 3 omega 1}
 normal{bump_map{jpeg grass_img2 bump_size 1} turbulence .5 lambda 3 omega 1}
 translate -.5 rotate 90*x scale 10
}
#else
#declare t_grass1=
texture{
 pigment{HuntersGreen*.5+Gold*.25}
 normal{granite scale .001}
}
#declare t_grass2=
texture{
 pigment{ForestGreen*.25+Gold*.5}
 normal{granite scale .001}
}
#end
#declare t_grass=
texture{
  slope y
  texture_map{
   [0 t_grass1]
   [.5 t_grass2]
  }
}

// slope y based texture
#declare t_terrain_slope=
texture{
  slope y
  texture_map{
   [0.00 t_rocks]
   [0.50 t_rocks]
   [green_level-green_trans t_rocks]
   [green_level t_grass]
   [1.00 t_grass]
  }
}

// final height based texture
#declare wl=Water_Height/terrain_size.y;
#declare t_terrain=
texture{
 pigment_pattern{gradient y scale <1,terrain_size.y,1>}
 texture_map{
  #if (use_water)
  [0 t_sand]
  [wl+.002 t_sand]
  [wl+.004 t_terrain_slope]
  #else
  [0 t_terrain_slope]
  #end
  [1  t_terrain_slope]
 }
}

// trees leaves texture
#declare t_trees_leaves=t_grass
#declare t_trees=
texture{
  pigment_pattern{cells turbulence .5 scale .25}
  texture_map{
   [0 t_trees_leaves]
   [.45 t_trees_leaves]
   [.55 pigment{rgbt 1}]
   [1 pigment{rgbt 1}]
  }
}
