/*
  Persistence of Vision Ray Tracer Scene Description File

  Articulated bird flock (borrowed for the sea bouy scene)

  Notes: 
  + body orientation along z, wings along x
  + now with patterned distribution
  
  --
  Jaime Vives Piqueres, May.2007
  
*/

// *********
// bird body
// *********
#declare bird_body=
blob{threshold .5
 sphere{0,.67,1}
 sphere{0,.5,1 translate -.25*z}
 sphere{0,.5,1 translate .25*z}
 sphere{0,.33,1 translate -.5*z}
 sphere{0,.25,1 translate .5*z}
 sphere{0,.33,1 translate -.75*z}
 sphere{0,.33,1 scale <1,.25,1> translate .75*z}
 sphere{0,.25,1 translate -.85*z}
 pigment{White}
}

// ****************
// articulated wing
// ****************
#macro bird_wing(wing_pos)

#local r_wing_pos=seed(wing_pos);
#local this_wing=
union{
 difference{
  sphere{0,.5 scale <4,.05,.75>}
  plane{x,0}
  plane{x,1 inverse}
 }
 difference{
 sphere{0,.5 scale <4,.05,.5>}
  plane{x,1}
  translate -1*x rotate (-45*rand(r_wing_pos))*y translate 1*x
 }
 pigment{White}
 translate <.25,0,-.2>
 rotate (-60+120*rand(r_wing_pos))*z
}

object{this_wing}

#end // of macro


// **********************
// articulated bird macro
// **********************
#macro articulated_bird(bird_pos)

#local r_bird_pos=seed(bird_pos);
#local wing_pos=1000*rand(r_bird_pos); // seed for the wings pose
#local this_bird=
union{
 object{bird_body}
 object{bird_wing(wing_pos)}
 object{bird_wing(wing_pos) scale <-1,1,1>}
 rotate (-45+90*rand(r_bird_pos))*x // up/down flight
 rotate (-45+90*rand(r_bird_pos))*z // left/right flight
}

object{this_bird}

#end // of macro


// ********************
// whole flock of birds
// ********************
#declare f_flock= // flock distribution
function{
 pattern{
  bumps 
  turbulence .33 
  scale flock_area.x*.2
  translate flock_area.x*20*rand(r_pos)
 }
}
#declare i_flock=0;
#declare flock=
union{
#while (i_flock<n_flock)
 #declare flkx=-flock_area.x*.5+flock_area.x*rand(r_pos);
 #declare flky=flock_area.y*rand(r_pos);
 #declare flkz=-flock_area.z*.5+flock_area.z*rand(r_pos);
 #if (rand(r_pos)>.01)
  // with distribution function
  #if (f_flock(flkx,flky,flkz)>.75)
   object{articulated_bird(1000*rand(r_pos))
    scale 1
    translate <flkx,flky,flkz>
   }
   #declare i_flock=i_flock+1;
  #end
 #else
  // random
   object{articulated_bird(1000*rand(r_pos))
    scale 1
    translate <flkx,flky,flkz>
   }
   #declare i_flock=i_flock+1;
 #end
#end
}


